@@ -6,25 +6,28 @@
 
 #pragma once
 
-#include "OdrsReviewsBackend.h"
 #include "discovercommon_export.h"
+
+#include <AppStreamQt/pool.h>
+#include <AppStreamQt/release.h>
 #include <KOSRelease>
 #include <QObject>
 
+#include <optional>
+
 class DISCOVERCOMMON_EXPORT AppStreamIntegration : public QObject
 {
     Q_OBJECT
 public:
     static AppStreamIntegration *global();
 
-    QSharedPointer<OdrsReviewsBackend> reviews();
+    std::optional<AppStream::Release> getDistroUpgrade(AppStream::Pool *pool, std::optional<QString> IdOverride = {});
     KOSRelease *osRelease()
     {
         return &m_osrelease;
     }
 
 private:
-    QSharedPointer<OdrsReviewsBackend> m_reviews;
     KOSRelease m_osrelease;
 
     AppStreamIntegration()
