@@ -5,8 +5,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef FLATPAKSOURCESBACKEND_H
-#define FLATPAKSOURCESBACKEND_H
+#pragma once
 
 #include <QStack>
 #include <QStandardItemModel>
@@ -26,6 +25,7 @@ public:
 
     enum Roles {
         IconUrlRole = LastRole + 1,
+        PrioRole,
     };
 
     QAbstractItemModel *sources() override;
@@ -52,22 +52,22 @@ public:
     int originIndex(const QString &sourceId) const;
     QStandardItem *sourceByUrl(const QString &url) const;
     QStandardItem *sourceById(const QString &sourceId) const;
+    DiscoverAction *inlineAction() const override
+    {
+        return m_saveAction;
+    }
 
     void cancel() override;
     void proceed() override;
 
-    static void populateRemote(FlatpakRemote *remote, const QString &name, const QString &url, const QString &gpgkey);
-    void checkRepositories(const QStringList &repoNames);
-
-private:
-    bool listRepositories(FlatpakInstallation *installation);
+    void save();
     void addRemote(FlatpakRemote *remote, FlatpakInstallation *installation);
 
+private:
     FlatpakInstallation *m_preferredInstallation;
     QStandardItemModel *m_sources;
     DiscoverAction *const m_flathubAction;
+    DiscoverAction *const m_saveAction;
     QStandardItem *m_noSourcesItem;
     QStack<std::function<void()>> m_proceedFunctions;
 };
-
-#endif // FLATPAKSOURCESBACKEND_H
