@@ -23,10 +23,13 @@
 #include <KLocalizedString>
 #include <KSharedConfig>
 
+#include <algorithm>
 #include <optional>
 
-#include "libdiscover_backend_packagekit_debug.h"
+#if !QPK_CHECK_VERSION(1, 1, 4)
 #include "pk-offline-private.h"
+#endif
+#include "libdiscover_backend_packagekit_debug.h"
 #include "utils.h"
 
 using namespace Qt::StringLiterals;
@@ -190,16 +193,9 @@ public:
     QString longDescription() override
     {
         QStringList changes;
-        auto resources = withoutDuplicates();
 
-        {
-            QCollator collator;
-            collator.setCaseSensitivity(Qt::CaseInsensitive);
-            const auto sortUpdateItems = [&collator](auto *a, auto *b) -> bool {
-                return collator(a->name(), b->name());
-            };
-            std::sort(resources.begin(), resources.end(), sortUpdateItems);
-        }
+        auto resources = withoutDuplicates();
+        std::ranges::sort(resources, std::less{}, &PackageKitResource::nameSortKey);
 
         for (auto resource : std::as_const(resources)) {
             const auto changelog = resource->changelog();
@@ -349,7 +345,13 @@ void PackageKitUpdater::prepare()
         return;
     }
 
+#if QPK_CHECK_VERSION(1, 1, 4)
+    auto results = offline->getResults();
+    results.waitForFinished();
+    if (results.isError() || !results.success()) {
+#else
     if (QFile::exists(QStringLiteral(PK_OFFLINE_RESULTS_FILENAME))) {
+#endif
         qCDebug(LIBDISCOVER_BACKEND_PACKAGEKIT_LOG) << "Removed offline results file";
         offline->clearResults();
     }
@@ -933,5 +935,10 @@ void PackageKitUpdater::setOfflineUpdateAction(PackageKit::Offline::Action actio
     }
 }
 
+bool PackageKitUpdater::isFetchingUpdates() const
+{
+    return m_backend->refresher() != nullptr;
+}
+
 #include "PackageKitUpdater.moc"
 #include "moc_PackageKitUpdater.cpp"
